@@ -20,6 +20,9 @@
  */
 
 #include <sys/types.h>
+#ifndef __OpenBSD__
+#include <sys/sysmacros.h>
+#endif
 #include <sys/mman.h>
 #include <fcntl.h>
 #include <sys/stat.h>
@@ -35,7 +38,11 @@
 #include "libv4lcontrol.h"
 #include "libv4lcontrol-priv.h"
 #include "../libv4lsyscall-priv.h"
+#if defined(__OpenBSD__)
+#include <sys/videoio.h>
+#else
 #include <linux/videodev2.h>
+#endif
 
 #define ARRAY_SIZE(x) ((int)sizeof(x) / (int)sizeof((x)[0]))
 
@@ -72,8 +79,8 @@ static const struct v4lcontrol_flags_info v4lcontrol_flags[] = {
 	{ 0x046d, 0x09b2, 0, "FUJITSU", "FJNB1C9",
 		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED, 0,
 		"FUJITSU", "LifeBook P7230" },
-	/* Note no whitespace padding for board vendor, this is not a typo */
-	{ 0x04f2, 0xb012, 0, "PEGATRON CORPORATION", "X71TL     ",
+	/* A re-branded ASUS notebook */
+	{ 0x04f2, 0xb012, 0, "Founder PC", "T14MF",
 		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED },
 	/* These 3 PACKARD BELL's seem to be Asus notebook in disguise */
 	{ 0x04f2, 0xb012, 0, "Packard Bell BV", "T32A      ",
@@ -86,6 +93,10 @@ static const struct v4lcontrol_flags_info v4lcontrol_flags[] = {
 		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED },
 	{ 0x04f2, 0xb012, 0, "PACKARD BELL BV              ", "EasyNote_F0945",
 		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED },
+	/* This Asus has its camera the right way up, so we've an entry here
+	   to override the wildcard match from the upside_down table. */
+	{ 0x04f2, 0xb012, 0, "ASUSTeK Computer Inc.        ", "F3Sc      ",
+		0 },
 	{ 0x04f2, 0xb071, 0, "AXIOO", "PICO DJH Model",
 		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED },
 	{ 0x04f2, 0xb071, 0, "PEGATRON CORPORATION", "H54",
@@ -129,9 +140,14 @@ static const struct v4lcontrol_flags_info v4lcontrol_flags[] = {
 	{ 0x04f2, 0xb213, 0, "FUJITSU", "FJNBB16",
 		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED, 0,
 		"FUJITSU", "LIFEBOOK LH531" },
+	{ 0x04f2, 0xb213, 0, "FUJITSU", "FJNBB18",
+		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED, 0,
+		"FUJITSU", "FMVN77ED" },
 	{ 0x04f2, 0xb213, 0, "FUJITSU", "FJNBB19",
 		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED, 0,
 		"FUJITSU", "LIFEBOOK NH751" },
+	{ 0x04f2, 0xb217, 0, "LENOVO", "42982YG",
+		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED },
 	{ 0x04f2, 0xb217, 0, "LENOVO", "42992QG",
 		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED },
 	{ 0x04f2, 0xb27c, 0, "LENOVO", "12973MG",
@@ -140,10 +156,6 @@ static const struct v4lcontrol_flags_info v4lcontrol_flags[] = {
 	{ 0x064e, 0xa111, 0, "Acer, Inc.", "Prespa1         ", 
 		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED, 0,
 		"Acer, inc.", "Aspire 5570     " },
-	{ 0x064e, 0xa111, 0, "PEGATRON CORPORATION         ", "F5C     ",
-		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED },
-	{ 0x064e, 0xa111, 0, "PEGATRON CORPORATION         ", "F5SR    ",
-		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED },
 	/* 2 reports:
 	   Unknown laptop model -> System Vendor: "  IDEALMAX"
 	   Síragon SL-6120      -> System Vendor: "PEGA PC"
@@ -153,8 +165,6 @@ static const struct v4lcontrol_flags_info v4lcontrol_flags[] = {
 		NULL, "H34" },
 	{ 0x064e, 0xa212, 0, "MEDIONAG", "WeTab ",
 		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED },
-	{ 0x174f, 0x5a35, 0, "PEGATRON CORPORATION         ", "F5SL    ",
-		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED },
 	{ 0x174f, 0x6a51, 0, NULL, "S96S",
 		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED, 0,
 		"MicroLink", "S96S" },
@@ -208,6 +218,8 @@ static const struct v4lcontrol_flags_info v4lcontrol_flags[] = {
 		V4LCONTROL_ROTATED_90_JPEG | V4LCONTROL_WANTS_WB, 1500 },
 	{ 0x145f, 0x013c, 0,    NULL, NULL,
 		V4LCONTROL_ROTATED_90_JPEG | V4LCONTROL_WANTS_WB, 1500 },
+	{ 0x1ae7, 0x2001, 0,    NULL, NULL,
+		V4LCONTROL_ROTATED_90_JPEG | V4LCONTROL_WANTS_WB, 1500 },
 	/* Pac7311 based devices */
 	{ 0x093a, 0x2600, 0x0f, NULL, NULL, V4LCONTROL_WANTS_WB },
 	/* sq905 devices */
@@ -250,18 +262,19 @@ static const struct v4lcontrol_flags_info v4lcontrol_flags[] = {
 	{ 0x1046, 0x9967, 0,    NULL, NULL, V4LCONTROL_FORCE_TINYJPEG },
 };
 
-static const struct v4l2_queryctrl fake_controls[];
+static const struct v4l2_queryctrl fake_controls[V4LCONTROL_COUNT];
 
 static const char *asus_board_vendor[] = {
 	"ASUSTeK Computer Inc.",
 	"ASUSTeK Computer INC.",
 	"ASUS CORPORATION",
+	"PEGATRON CORPORATION",
 	NULL };
 
 static const char *asus_board_name[] = {
-	"A3[A-Z]*",
+	"A3[A-Z]*", "A6R*", "A7J", "A7M", "A7Sn",
 	"B50[A-Z]*",
-	"F[3579][A-Z]*", "F70[A-Z]*", "F[58]2[A-Z]*",
+	"F[23579][A-Z]*", "F70[A-Z]*", "F[58]2[A-Z]*",
 	"G[12][A-Z]*", "G[57]0[A-Z]*",
 	"K[4567]0[A-Z]*", "K[56]1[A-Z]*", "K52[A-Z]*", "K[45]3[A-Z]*",
 	"N[12579]0[A-Z]*", "N[56]1[A-Z]*", "N82[A-Z]*", "N[47]3[A-Z]*",
@@ -284,6 +297,7 @@ static const struct v4lcontrol_usb_id asus_camera_id[] = {
 	{ 0x04f2, 0xb071 },
 	{ 0x04f2, 0xb072 },
 	{ 0x04f2, 0xb106 },
+	{ 0x04f2, 0xb169 },
 	{ 0x04f2, 0xb16b },
 	{ 0x04f2, 0xb1b9 },
 	{ 0x04f2, 0xb1bb },
@@ -306,6 +320,7 @@ static const struct v4lcontrol_usb_id asus_camera_id[] = {
 	{ 0x174f, 0x8a31 },
 	{ 0x174f, 0xa311 },
 	{ 0x1d4d, 0x1002 },
+	{ 0x05e1, 0x0501 },
 	{ 0x0000, 0x0000 }
 };
 
@@ -346,11 +361,18 @@ static int v4lcontrol_get_usb_info(struct v4lcontrol_data *data,
 		int *speed)
 {
 	FILE *f;
-	int i, minor;
+	int i, minor_dev;
 	struct stat st;
 	char sysfs_name[512];
 	char c, *s, buf[32];
 
+	snprintf(sysfs_name, sizeof(sysfs_name),
+	    "%s/sys/class/video4linux", sysfs_prefix);
+
+	/* Check for sysfs mounted before trying to search */
+	if (stat(sysfs_name, &st) != 0)
+		return 0; /* Not found, sysfs not mounted? */
+
 	if (fstat(data->fd, &st) || !S_ISCHR(st.st_mode))
 		return 0; /* Should never happen */
 
@@ -365,8 +387,8 @@ static int v4lcontrol_get_usb_info(struct v4lcontrol_data *data,
 		s = fgets(buf, sizeof(buf), f);
 		fclose(f);
 
-		if (s && sscanf(buf, "%*d:%d%c", &minor, &c) == 2 &&
-		    c == '\n' && minor == minor(st.st_rdev))
+		if (s && sscanf(buf, "%*d:%d%c", &minor_dev, &c) == 2 &&
+		    c == '\n' && minor_dev == minor(st.st_rdev))
 			break;
 	}
 	if (i == 256)
@@ -449,6 +471,7 @@ static int find_dmi_string(const char **table_entries, const char *dmi_value)
 	const char **entry_ptr;
 	char *trimmed_dmi;
 	size_t n;
+	int found = 0;
 
 	if (!start) return 0;
 
@@ -456,16 +479,19 @@ static int find_dmi_string(const char **table_entries, const char *dmi_value)
 	while (isspace(*start)) start++;
 	n = strlen(start);
 	while (n > 0 && isspace(start[n-1])) --n;
-	trimmed_dmi = strndupa(start, n);
+	trimmed_dmi = strndup(start, n);
 
 	/* find trimmed value */
 	for (entry_ptr = table_entries; *entry_ptr;  entry_ptr++) {
-		const int found = fnmatch(*entry_ptr, trimmed_dmi, 0) == 0;
+		found = fnmatch(*entry_ptr, trimmed_dmi, 0) == 0;
 		/* fprintf(stderr, "find_dmi_string('%s', '%s'->'%s')=%i\n", *entry_ptr, dmi_value, trimmed_dmi, found); */
 		if (found)
-			return 1;
+			break;
 	}
-	return 0;
+
+	free(trimmed_dmi);
+
+	return found;
 }
 
 /*
@@ -505,15 +531,6 @@ static void v4lcontrol_get_flags_from_db(struct v4lcontrol_data *data,
 	v4lcontrol_get_dmi_string(sysfs_prefix, "board_version", dmi_board_version,
 			sizeof(dmi_board_version));
 
-	for (i = 0; i < ARRAY_SIZE(upside_down); i++)
-		if (find_dmi_string(upside_down[i].board_vendor, dmi_board_vendor) &&
-		    find_dmi_string(upside_down[i].board_name, dmi_board_name) &&
-		    find_usb_id(upside_down[i].camera_id, vendor_id, product_id)) {
-			/* found entry */
-			data->flags |= V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED;
-			break;
-		}
- 
 	for (i = 0; i < ARRAY_SIZE(v4lcontrol_flags); i++)
 		if (v4lcontrol_flags[i].vendor_id == vendor_id &&
 				v4lcontrol_flags[i].product_id ==
@@ -534,11 +551,23 @@ static void v4lcontrol_get_flags_from_db(struct v4lcontrol_data *data,
 				 !strcmp(v4lcontrol_flags[i].dmi_board_version, dmi_board_version))) {
 			data->flags |= v4lcontrol_flags[i].flags;
 			data->flags_info = &v4lcontrol_flags[i];
+			/* Entries in the v4lcontrol_flags table override
+			   wildcard matches in the upside_down table. */
+			return;
+		}
+
+	for (i = 0; i < ARRAY_SIZE(upside_down); i++)
+		if (find_dmi_string(upside_down[i].board_vendor, dmi_board_vendor) &&
+		    find_dmi_string(upside_down[i].board_name, dmi_board_name) &&
+		    find_usb_id(upside_down[i].camera_id, vendor_id, product_id)) {
+			/* found entry */
+			data->flags |= V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED;
 			break;
 		}
 }
 
-struct v4lcontrol_data *v4lcontrol_create(int fd, int always_needs_conversion)
+struct v4lcontrol_data *v4lcontrol_create(int fd, void *dev_ops_priv,
+	const struct libv4l_dev_ops *dev_ops, int always_needs_conversion)
 {
 	int shm_fd;
 	int i, rc, got_usb_info, speed, init = 0;
@@ -558,10 +587,14 @@ struct v4lcontrol_data *v4lcontrol_create(int fd, int always_needs_conversion)
 	}
 
 	data->fd = fd;
+	data->dev_ops = dev_ops;
+	data->dev_ops_priv = dev_ops_priv;
 
 	/* Check if the driver has indicated some form of flipping is needed */
-	if ((SYS_IOCTL(data->fd, VIDIOC_G_INPUT, &input.index) == 0) &&
-			(SYS_IOCTL(data->fd, VIDIOC_ENUMINPUT, &input) == 0)) {
+	if ((data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+				  VIDIOC_G_INPUT, &input.index) == 0) &&
+	    (data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+	    			VIDIOC_ENUMINPUT, &input) == 0)) {
 		if (input.status & V4L2_IN_ST_HFLIP)
 			data->flags |= V4LCONTROL_HFLIPPED;
 		if (input.status & V4L2_IN_ST_VFLIP)
@@ -599,7 +632,8 @@ struct v4lcontrol_data *v4lcontrol_create(int fd, int always_needs_conversion)
 		data->flags = strtol(s, NULL, 0);
 
 	ctrl.id = V4L2_CTRL_FLAG_NEXT_CTRL;
-	if (SYS_IOCTL(data->fd, VIDIOC_QUERYCTRL, &ctrl) == 0)
+	if (data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+			VIDIOC_QUERYCTRL, &ctrl) == 0)
 		data->priv_flags |= V4LCONTROL_SUPPORTS_NEXT_CTRL;
 
 	/* If the device always needs conversion, we can add fake controls at no cost
@@ -607,8 +641,11 @@ struct v4lcontrol_data *v4lcontrol_create(int fd, int always_needs_conversion)
 	if (always_needs_conversion || v4lcontrol_needs_conversion(data)) {
 		for (i = 0; i < V4LCONTROL_AUTO_ENABLE_COUNT; i++) {
 			ctrl.id = fake_controls[i].id;
-			rc = SYS_IOCTL(data->fd, VIDIOC_QUERYCTRL, &ctrl);
-			if (rc == -1 || (rc == 0 && (ctrl.flags & V4L2_CTRL_FLAG_DISABLED)))
+			rc = data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+					VIDIOC_QUERYCTRL, &ctrl);
+			if (rc == -1 ||
+			    (rc == 0 &&
+			     (ctrl.flags & V4L2_CTRL_FLAG_DISABLED)))
 				data->controls |= 1 << i;
 		}
 	}
@@ -619,17 +656,20 @@ struct v4lcontrol_data *v4lcontrol_create(int fd, int always_needs_conversion)
 	   different sensors with / without autogain or the necessary controls. */
 	while (data->flags & V4LCONTROL_WANTS_AUTOGAIN) {
 		ctrl.id = V4L2_CID_AUTOGAIN;
-		rc = SYS_IOCTL(data->fd, VIDIOC_QUERYCTRL, &ctrl);
+		rc = data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+				VIDIOC_QUERYCTRL, &ctrl);
 		if (rc == 0 && !(ctrl.flags & V4L2_CTRL_FLAG_DISABLED))
 			break;
 
 		ctrl.id = V4L2_CID_EXPOSURE;
-		rc = SYS_IOCTL(data->fd, VIDIOC_QUERYCTRL, &ctrl);
+		rc = data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+				VIDIOC_QUERYCTRL, &ctrl);
 		if (rc != 0 || (ctrl.flags & V4L2_CTRL_FLAG_DISABLED))
 			break;
 
 		ctrl.id = V4L2_CID_GAIN;
-		rc = SYS_IOCTL(data->fd, VIDIOC_QUERYCTRL, &ctrl);
+		rc = data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+				VIDIOC_QUERYCTRL, &ctrl);
 		if (rc != 0 || (ctrl.flags & V4L2_CTRL_FLAG_DISABLED))
 			break;
 
@@ -646,7 +686,8 @@ struct v4lcontrol_data *v4lcontrol_create(int fd, int always_needs_conversion)
 	if (data->controls == 0)
 		return data; /* No need to create a shared memory segment */
 
-	if (SYS_IOCTL(fd, VIDIOC_QUERYCAP, &cap)) {
+	if (data->dev_ops->ioctl(data->dev_ops_priv, fd,
+			VIDIOC_QUERYCAP, &cap)) {
 		perror("libv4lcontrol: error querying device capabilities");
 		goto error;
 	}
@@ -674,6 +715,7 @@ struct v4lcontrol_data *v4lcontrol_create(int fd, int always_needs_conversion)
 		if (shm_name[i] == '/')
 			shm_name[i] = '-';
 
+#ifndef ANDROID
 	/* Open the shared memory object identified by shm_name */
 	shm_fd = shm_open(shm_name, (O_CREAT | O_EXCL | O_RDWR), (S_IREAD | S_IWRITE));
 	if (shm_fd >= 0)
@@ -683,19 +725,25 @@ struct v4lcontrol_data *v4lcontrol_create(int fd, int always_needs_conversion)
 
 	if (shm_fd >= 0) {
 		/* Set the shared memory size */
-		ftruncate(shm_fd, V4LCONTROL_SHM_SIZE);
+		int ret = ftruncate(shm_fd, V4LCONTROL_SHM_SIZE);
+		if (ret) {
+			perror("libv4lcontrol: shm ftruncate failed");
+			close(shm_fd);
+		} else {
+			/* Retrieve a pointer to the shm object */
+			data->shm_values = mmap(NULL, V4LCONTROL_SHM_SIZE,
+						PROT_READ | PROT_WRITE,
+						MAP_SHARED, shm_fd, 0);
+			close(shm_fd);
 
-		/* Retreive a pointer to the shm object */
-		data->shm_values = mmap(NULL, V4LCONTROL_SHM_SIZE, (PROT_READ | PROT_WRITE),
-				MAP_SHARED, shm_fd, 0);
-		close(shm_fd);
-
-		if (data->shm_values == MAP_FAILED) {
-			perror("libv4lcontrol: error shm mmap failed");
-			data->shm_values = NULL;
+			if (data->shm_values == MAP_FAILED) {
+				perror("libv4lcontrol: shm mmap failed");
+				data->shm_values = NULL;
+			}
 		}
 	} else
 		perror("libv4lcontrol: error creating shm segment failed");
+#endif
 
 	/* Fall back to malloc */
 	if (data->shm_values == NULL) {
@@ -746,7 +794,7 @@ static const struct v4l2_queryctrl fake_controls[V4LCONTROL_COUNT] = {
 	{
 		.id = V4L2_CID_AUTO_WHITE_BALANCE,
 		.type = V4L2_CTRL_TYPE_BOOLEAN,
-		.name =  "Whitebalance (software)",
+		.name =  "White Balance, Automatic",
 		.minimum = 0,
 		.maximum = 1,
 		.step = 1,
@@ -755,7 +803,7 @@ static const struct v4l2_queryctrl fake_controls[V4LCONTROL_COUNT] = {
 	}, {
 		.id = V4L2_CID_HFLIP,
 		.type = V4L2_CTRL_TYPE_BOOLEAN,
-		.name =  "Horizontal flip (sw)",
+		.name =  "Horizontal Flip",
 		.minimum = 0,
 		.maximum = 1,
 		.step = 1,
@@ -764,7 +812,7 @@ static const struct v4l2_queryctrl fake_controls[V4LCONTROL_COUNT] = {
 	}, {
 		.id = V4L2_CID_VFLIP,
 		.type = V4L2_CTRL_TYPE_BOOLEAN,
-		.name =  "Vertical flip (sw)",
+		.name =  "Vertical Flip",
 		.minimum = 0,
 		.maximum = 1,
 		.step = 1,
@@ -773,17 +821,17 @@ static const struct v4l2_queryctrl fake_controls[V4LCONTROL_COUNT] = {
 	}, {
 		.id = V4L2_CID_GAMMA,
 		.type = V4L2_CTRL_TYPE_INTEGER,
-		.name =  "Gamma (software)",
+		.name =  "Gamma",
 		.minimum = 500,  /* == 0.5 */
 		.maximum = 3000, /* == 3.0 */
 		.step = 1,
 		.default_value = 1000, /* == 1.0 */
-		.flags = 0
+		.flags = V4L2_CTRL_FLAG_SLIDER
 	}, { /* Dummy place holder for V4LCONTROL_AUTO_ENABLE_COUNT */
 	}, {
 		.id = V4L2_CID_AUTOGAIN,
 		.type = V4L2_CTRL_TYPE_BOOLEAN,
-		.name =  "Auto Gain (software)",
+		.name =  "Gain, Automatic",
 		.minimum = 0,
 		.maximum = 1,
 		.step = 1,
@@ -792,12 +840,12 @@ static const struct v4l2_queryctrl fake_controls[V4LCONTROL_COUNT] = {
 	}, {
 		.id = V4L2_CTRL_CLASS_USER + 0x2000, /* FIXME */
 		.type = V4L2_CTRL_TYPE_INTEGER,
-		.name =  "Auto Gain target",
+		.name =  "Auto Gain Target",
 		.minimum = 0,
 		.maximum = 255,
 		.step = 1,
 		.default_value = 100,
-		.flags = 0
+		.flags = V4L2_CTRL_FLAG_SLIDER
 	},
 };
 
@@ -832,12 +880,13 @@ int v4lcontrol_vidioc_queryctrl(struct v4lcontrol_data *data, void *arg)
 		}
 
 	/* find out what the kernel driver would respond. */
-	retval = SYS_IOCTL(data->fd, VIDIOC_QUERYCTRL, arg);
+	retval = data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+			VIDIOC_QUERYCTRL, arg);
 
 	if ((data->priv_flags & V4LCONTROL_SUPPORTS_NEXT_CTRL) &&
 			(orig_id & V4L2_CTRL_FLAG_NEXT_CTRL)) {
 		/* If the hardware has no more controls check if we still have any
-		   fake controls with a higher id then the hardware's highest */
+		   fake controls with a higher id than the hardware's highest */
 		if (retval)
 			ctrl->id = V4L2_CTRL_ID_MASK;
 
@@ -869,7 +918,83 @@ int v4lcontrol_vidioc_g_ctrl(struct v4lcontrol_data *data, void *arg)
 			return 0;
 		}
 
-	return SYS_IOCTL(data->fd, VIDIOC_G_CTRL, arg);
+	return data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+			VIDIOC_G_CTRL, arg);
+}
+
+static void v4lcontrol_alloc_valid_controls(struct v4lcontrol_data *data,
+			const struct v4l2_ext_controls *src,
+			struct v4l2_ext_controls *dst)
+{
+	struct v4l2_ext_control *ctrl;
+	unsigned i, j;
+
+	*dst = *src;
+	if (data->controls == 0)
+		return;
+	ctrl = malloc(src->count * sizeof(*ctrl));
+	if (ctrl == NULL)
+		return;
+	dst->controls = ctrl;
+	dst->count = 0;
+	for (i = 0; i < src->count; i++) {
+		for (j = 0; j < V4LCONTROL_COUNT; j++)
+			if ((data->controls & (1 << j)) &&
+			    src->controls[i].id == fake_controls[j].id)
+				break;
+		if (j == V4LCONTROL_COUNT)
+			ctrl[dst->count++] = src->controls[i];
+	}
+}
+
+static void v4lcontrol_free_valid_controls(struct v4lcontrol_data *data,
+			struct v4l2_ext_controls *src,
+			struct v4l2_ext_controls *dst)
+{
+	unsigned i, j, k = 0;
+	int inc_idx;
+
+	src->error_idx = dst->error_idx;
+	if (dst->controls == src->controls)
+		return;
+
+	inc_idx = dst->error_idx < dst->count;
+	for (i = 0; i < src->count; i++) {
+		for (j = 0; j < V4LCONTROL_COUNT; j++)
+			if ((data->controls & (1 << j)) &&
+			    src->controls[i].id == fake_controls[j].id)
+				break;
+		if (j == V4LCONTROL_COUNT)
+			src->controls[i] = dst->controls[k++];
+		else if (inc_idx)
+			src->error_idx++;
+	}
+	free(dst->controls);
+}
+
+int v4lcontrol_vidioc_g_ext_ctrls(struct v4lcontrol_data *data, void *arg)
+{
+	struct v4l2_ext_controls *ctrls = arg;
+	struct v4l2_ext_controls dst;
+	int i, j;
+	int res;
+
+	v4lcontrol_alloc_valid_controls(data, ctrls, &dst);
+	res = data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+			VIDIOC_G_EXT_CTRLS, &dst);
+	v4lcontrol_free_valid_controls(data, ctrls, &dst);
+	if (res)
+		return res;
+
+	for (i = 0; i < ctrls->count; i++) {
+		for (j = 0; j < V4LCONTROL_COUNT; j++)
+			if ((data->controls & (1 << j)) &&
+			    ctrls->controls[i].id == fake_controls[j].id) {
+				ctrls->controls[i].value = data->shm_values[j];
+				break;
+			}
+	}
+	return 0;
 }
 
 int v4lcontrol_vidioc_s_ctrl(struct v4lcontrol_data *data, void *arg)
@@ -890,7 +1015,74 @@ int v4lcontrol_vidioc_s_ctrl(struct v4lcontrol_data *data, void *arg)
 			return 0;
 		}
 
-	return SYS_IOCTL(data->fd, VIDIOC_S_CTRL, arg);
+	return data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+			VIDIOC_S_CTRL, arg);
+}
+
+static int v4lcontrol_validate_ext_ctrls(struct v4lcontrol_data *data,
+		struct v4l2_ext_controls *ctrls)
+{
+	int i, j;
+
+	if (data->controls == 0)
+		return 0;
+	for (i = 0; i < ctrls->count; i++) {
+		for (j = 0; j < V4LCONTROL_COUNT; j++)
+			if ((data->controls & (1 << j)) &&
+			    ctrls->controls[i].id == fake_controls[j].id) {
+				if (ctrls->controls[i].value > fake_controls[j].maximum ||
+				    ctrls->controls[i].value < fake_controls[j].minimum) {
+					ctrls->error_idx = i;
+					errno = EINVAL;
+					return -1;
+				}
+			}
+	}
+	return 0;
+}
+
+int v4lcontrol_vidioc_try_ext_ctrls(struct v4lcontrol_data *data, void *arg)
+{
+	struct v4l2_ext_controls *ctrls = arg;
+	struct v4l2_ext_controls dst;
+	int res = v4lcontrol_validate_ext_ctrls(data, ctrls);
+
+	if (res)
+		return res;
+
+	v4lcontrol_alloc_valid_controls(data, ctrls, &dst);
+	res = data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+			VIDIOC_TRY_EXT_CTRLS, &dst);
+	v4lcontrol_free_valid_controls(data, ctrls, &dst);
+	return res;
+}
+
+int v4lcontrol_vidioc_s_ext_ctrls(struct v4lcontrol_data *data, void *arg)
+{
+	struct v4l2_ext_controls *ctrls = arg;
+	struct v4l2_ext_controls dst;
+	int i, j;
+	int res = v4lcontrol_validate_ext_ctrls(data, ctrls);
+
+	if (res)
+		return res;
+
+	v4lcontrol_alloc_valid_controls(data, ctrls, &dst);
+	res = data->dev_ops->ioctl(data->dev_ops_priv, data->fd,
+			VIDIOC_S_EXT_CTRLS, &dst);
+	v4lcontrol_free_valid_controls(data, ctrls, &dst);
+	if (res)
+		return res;
+
+	for (i = 0; i < ctrls->count; i++) {
+		for (j = 0; j < V4LCONTROL_COUNT; j++)
+			if ((data->controls & (1 << j)) &&
+			    ctrls->controls[i].id == fake_controls[j].id) {
+				data->shm_values[j] = ctrls->controls[i].value;
+				break;
+			}
+	}
+	return 0;
 }
 
 int v4lcontrol_get_bandwidth(struct v4lcontrol_data *data)
