@@ -20,8 +20,6 @@
 #ifndef QV4L2_H
 #define QV4L2_H
 
-#include <config.h>
-
 #include <QMainWindow>
 #include <QTabWidget>
 #include <QSignalMapper>
@@ -37,7 +35,7 @@
 #include <libv4l2.h>
 
 extern "C" {
-#include <vivid-tpg.h>
+#include <v4l2-tpg.h>
 }
 #include "cv4l-helpers.h"
 #include "raw2sliced.h"
@@ -51,10 +49,10 @@ class VbiTab;
 class QCloseEvent;
 class CaptureWin;
 
-typedef std::vector<unsigned> ClassIDVec;
-typedef std::map<unsigned, ClassIDVec> ClassMap;
-typedef std::map<unsigned, struct v4l2_query_ext_ctrl> CtrlMap;
-typedef std::map<unsigned, QWidget *> WidgetMap;
+using ClassIDVec = std::vector<unsigned>;
+using ClassMap = std::map<unsigned, ClassIDVec>;
+using CtrlMap = std::map<unsigned, v4l2_query_ext_ctrl>;
+using WidgetMap = std::map<unsigned, QWidget *>;
 
 enum {
 	CTRL_UPDATE_ON_CHANGE = 0x10,
