@@ -16,37 +16,25 @@
     Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
  */
 
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <inttypes.h>
-#include <getopt.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <errno.h>
-#include <sys/ioctl.h>
-#include <sys/time.h>
-#include <math.h>
+#include <cctype>
+#include <cerrno>
+#include <cstring>
+#include <string>
+#include <vector>
 
-#ifdef ANDROID
-#include <android-config.h>
-#else
-#include <config.h>
-#endif
+#include <fcntl.h>
+#include <getopt.h>
+#include <sys/ioctl.h>
+#include <unistd.h>
 
 #ifdef HAVE_SYS_KLOG_H
 #include <sys/klog.h>
 #endif
 
 #include <linux/videodev2.h>
+#include <v4l-getsubopt.h>
 
-#include <list>
-#include <vector>
-#include <map>
-#include <string>
+#include <v4l2-info.h>
 
 #include "v4l2-dbg-bttv.h"
 #include "v4l2-dbg-saa7134.h"
@@ -55,66 +43,58 @@
 #include "v4l2-dbg-tvp5150.h"
 #include "v4l2-dbg-micron.h"
 
-#define ARRAY_SIZE(arr) ((int)(sizeof(arr) / sizeof((arr)[0])))
-
 struct board_list {
 	const char *name;
 	int prefix; 		/* Register prefix size */
-	const struct board_regs *regs;
-	int regs_size;
-	const struct board_regs *alt_regs;
-	int alt_regs_size;
+	const std::vector<board_regs> &regs;
+	const std::vector<board_regs> &alt_regs;
 };
 
-static const struct board_list boards[] = {
+static const std::vector<board_regs> empty{};
+
+static const std::vector<board_list> boards{
 #define AC97_BOARD 0
-	{				/* From v4l2-dbg-ac97.h */
+	{
+		/* From v4l2-dbg-ac97.h */
 		AC97_IDENT,
 		sizeof(AC97_PREFIX) - 1,
 		ac97_regs,
-		ARRAY_SIZE(ac97_regs),
-		NULL,
-		0,
+		empty,
 	},
-	{				/* From v4l2-dbg-bttv.h */
+	{
+		/* From v4l2-dbg-bttv.h */
 		BTTV_IDENT,
 		sizeof(BTTV_PREFIX) - 1,
 		bt8xx_regs,
-		ARRAY_SIZE(bt8xx_regs),
 		bt8xx_regs_other,
-		ARRAY_SIZE(bt8xx_regs_other),
 	},
-	{				/* From v4l2-dbg-saa7134.h */
+	{
+		/* From v4l2-dbg-saa7134.h */
 		SAA7134_IDENT,
 		sizeof(SAA7134_PREFIX) - 1,
 		saa7134_regs,
-		ARRAY_SIZE(saa7134_regs),
-		NULL,
-		0,
+		empty,
 	},
-	{				/* From v4l2-dbg-em28xx.h */
+	{
+		/* From v4l2-dbg-em28xx.h */
 		EM28XX_IDENT,
 		sizeof(EM28XX_PREFIX) - 1,
 		em28xx_regs,
-		ARRAY_SIZE(em28xx_regs),
 		em28xx_alt_regs,
-		ARRAY_SIZE(em28xx_alt_regs),
 	},
-	{				/* From v4l2-dbg-tvp5150.h */
+	{
+		/* From v4l2-dbg-tvp5150.h */
 		TVP5150_IDENT,
 		sizeof(TVP5150_PREFIX) - 1,
 		tvp5150_regs,
-		ARRAY_SIZE(tvp5150_regs),
-		NULL,
-		0,
+		empty,
 	},
-	{				/* From v4l2-dbg-micron.h */
+	{
+		/* From v4l2-dbg-micron.h */
 		MT9V011_IDENT,
 		sizeof(MT9V011_PREFIX) - 1,
 		mt9v011_regs,
-		ARRAY_SIZE(mt9v011_regs),
-		NULL,
-		0,
+		empty,
 	},
 };
 
@@ -147,116 +127,70 @@ static char options[OptLast];
 static unsigned capabilities;
 
 static struct option long_options[] = {
-	{"device", required_argument, 0, OptSetDevice},
-	{"help", no_argument, 0, OptHelp},
-	{"list-registers", optional_argument, 0, OptListRegisters},
-	{"get-register", required_argument, 0, OptGetRegister},
-	{"set-register", required_argument, 0, OptSetRegister},
-	{"chip", required_argument, 0, OptChip},
-	{"scan-chips", no_argument, 0, OptScanChips},
-	{"info", no_argument, 0, OptGetDriverInfo},
-	{"verbose", no_argument, 0, OptVerbose},
-	{"log-status", no_argument, 0, OptLogStatus},
-	{"list-symbols", no_argument, 0, OptListSymbols},
-	{"wide", required_argument, 0, OptSetStride},
-	{0, 0, 0, 0}
+	{"device", required_argument, nullptr, OptSetDevice},
+	{"help", no_argument, nullptr, OptHelp},
+	{"list-registers", optional_argument, nullptr, OptListRegisters},
+	{"get-register", required_argument, nullptr, OptGetRegister},
+	{"set-register", required_argument, nullptr, OptSetRegister},
+	{"chip", required_argument, nullptr, OptChip},
+	{"scan-chips", no_argument, nullptr, OptScanChips},
+	{"info", no_argument, nullptr, OptGetDriverInfo},
+	{"verbose", no_argument, nullptr, OptVerbose},
+	{"log-status", no_argument, nullptr, OptLogStatus},
+	{"list-symbols", no_argument, nullptr, OptListSymbols},
+	{"wide", required_argument, nullptr, OptSetStride},
+	{nullptr, 0, nullptr, 0}
 };
 
-static void usage(void)
+static void usage()
 {
 	printf("Usage: v4l2-dbg [options] [values]\n"
 	       "  -D, --info         Show driver info [VIDIOC_QUERYCAP]\n"
-	       "  -d, --device=<dev> Use device <dev> instead of /dev/video0\n"
+	       "  -d, --device <dev> Use device <dev> instead of /dev/video0\n"
 	       "                     If <dev> starts with a digit, then /dev/video<dev> is used\n"
 	       "  -h, --help         Display this help message\n"
 	       "  --verbose          Turn on verbose ioctl error reporting\n"
-	       "  -c, --chip=<chip>  The chip identifier to use with other commands\n"
+	       "  -c, --chip <chip>  The chip identifier to use with other commands\n"
 	       "                     It can be one of:\n"
 	       "                         bridge<num>: bridge chip number <num>\n"
 	       "                         bridge (default): same as bridge0\n"
 	       "                         subdev<num>: sub-device number <num>\n"
 	       "  -l, --list-registers[=min=<addr>[,max=<addr>]]\n"
 	       "		     Dump registers from <min> to <max> [VIDIOC_DBG_G_REGISTER]\n"
-	       "  -g, --get-register=<addr>\n"
+	       "  -g, --get-register <addr>\n"
 	       "		     Get the specified register [VIDIOC_DBG_G_REGISTER]\n"
-	       "  -s, --set-register=<addr>\n"
+	       "  -s, --set-register <addr>\n"
 	       "		     Set the register with the commandline arguments\n"
 	       "                     The register will autoincrement [VIDIOC_DBG_S_REGISTER]\n"
 	       "  -n, --scan-chips   Scan the available bridge and subdev chips [VIDIOC_DBG_G_CHIP_INFO]\n"
-	       "  -w, --wide=<reg length>\n"
+	       "  -w, --wide <reg length>\n"
 	       "		     Sets step between two registers\n"
 	       "  --list-symbols     List the symbolic register names you can use, if any\n"
 	       "  --log-status       Log the board status in the kernel log [VIDIOC_LOG_STATUS]\n");
-	exit(0);
-}
-
-static std::string cap2s(unsigned cap)
-{
-	std::string s;
-
-	if (cap & V4L2_CAP_VIDEO_CAPTURE)
-		s += "\t\tVideo Capture\n";
-	if (cap & V4L2_CAP_VIDEO_CAPTURE_MPLANE)
-		s += "\t\tVideo Capture Multiplanar\n";
-	if (cap & V4L2_CAP_VIDEO_OUTPUT)
-		s += "\t\tVideo Output\n";
-	if (cap & V4L2_CAP_VIDEO_OUTPUT_MPLANE)
-		s += "\t\tVideo Output Multiplanar\n";
-	if (cap & V4L2_CAP_VIDEO_M2M)
-		s += "\t\tVideo Memory-to-Memory\n";
-	if (cap & V4L2_CAP_VIDEO_M2M_MPLANE)
-		s += "\t\tVideo Memory-to-Memory Multiplanar\n";
-	if (cap & V4L2_CAP_VIDEO_OVERLAY)
-		s += "\t\tVideo Overlay\n";
-	if (cap & V4L2_CAP_VIDEO_OUTPUT_OVERLAY)
-		s += "\t\tVideo Output Overlay\n";
-	if (cap & V4L2_CAP_VBI_CAPTURE)
-		s += "\t\tVBI Capture\n";
-	if (cap & V4L2_CAP_VBI_OUTPUT)
-		s += "\t\tVBI Output\n";
-	if (cap & V4L2_CAP_SLICED_VBI_CAPTURE)
-		s += "\t\tSliced VBI Capture\n";
-	if (cap & V4L2_CAP_SLICED_VBI_OUTPUT)
-		s += "\t\tSliced VBI Output\n";
-	if (cap & V4L2_CAP_RDS_CAPTURE)
-		s += "\t\tRDS Capture\n";
-	if (cap & V4L2_CAP_RDS_OUTPUT)
-		s += "\t\tRDS Output\n";
-	if (cap & V4L2_CAP_SDR_CAPTURE)
-		s += "\t\tSDR Capture\n";
-	if (cap & V4L2_CAP_TUNER)
-		s += "\t\tTuner\n";
-	if (cap & V4L2_CAP_HW_FREQ_SEEK)
-		s += "\t\tHW Frequency Seek\n";
-	if (cap & V4L2_CAP_MODULATOR)
-		s += "\t\tModulator\n";
-	if (cap & V4L2_CAP_AUDIO)
-		s += "\t\tAudio\n";
-	if (cap & V4L2_CAP_RADIO)
-		s += "\t\tRadio\n";
-	if (cap & V4L2_CAP_READWRITE)
-		s += "\t\tRead/Write\n";
-	if (cap & V4L2_CAP_ASYNCIO)
-		s += "\t\tAsync I/O\n";
-	if (cap & V4L2_CAP_STREAMING)
-		s += "\t\tStreaming\n";
-	if (cap & V4L2_CAP_EXT_PIX_FORMAT)
-		s += "\t\tExtended Pix Format\n";
-	if (cap & V4L2_CAP_DEVICE_CAPS)
-		s += "\t\tDevice Capabilities\n";
-	return s;
 }
 
 static void print_regs(int fd, struct v4l2_dbg_register *reg, unsigned long min, unsigned long max, int stride)
 {
-	unsigned long mask = stride > 1 ? 0x1f : 0x0f;
+	unsigned long mask;
 	unsigned long i;
 	int line = 0;
 
+	/* Query size of the first register */
+	reg->reg = min;
+	if (ioctl(fd, VIDIOC_DBG_G_REGISTER, reg) == 0) {
+		/* If size is set, then use this as the stride */
+		if (reg->size)
+			stride = reg->size;
+	}
+
+	mask = stride > 2 ? 0x1f : 0x0f;
+
 	for (i = min & ~mask; i <= max; i += stride) {
 		if ((i & mask) == 0 && line % 32 == 0) {
 			if (stride == 4)
 				printf("\n                00       04       08       0C       10       14       18       1C");
+			else if (stride == 2)
+				printf("\n            00   02   04   06   08   0A   0C   0E");
 			else
 				printf("\n          00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F");
 		}
@@ -273,9 +207,8 @@ static void print_regs(int fd, struct v4l2_dbg_register *reg, unsigned long min,
 		if (ioctl(fd, VIDIOC_DBG_G_REGISTER, reg) < 0) {
 			perror("ioctl: VIDIOC_DBG_G_REGISTER failed\n");
 			break;
-		} else {
-			printf("%0*llx ", 2 * stride, reg->val);
 		}
+		printf("%0*llx ", 2 * stride, reg->val);
 		usleep(1);
 	}
 	printf("\n");
@@ -291,35 +224,35 @@ static void print_name(struct v4l2_dbg_chip_info *chip)
 static unsigned long long parse_reg(const struct board_list *curr_bd, const std::string &reg)
 {
 	if (curr_bd) {
-		for (int i = 0; i < curr_bd->regs_size; i++) {
-			if (!strcasecmp(reg.c_str(), curr_bd->regs[i].name) ||
-			    !strcasecmp(reg.c_str(), curr_bd->regs[i].name + curr_bd->prefix)) {
-				return curr_bd->regs[i].reg;
+		for (const auto &curr : curr_bd->regs) {
+			if (!strcasecmp(reg.c_str(), curr.name) ||
+			    !strcasecmp(reg.c_str(), curr.name + curr_bd->prefix)) {
+				return curr.reg;
 			}
 		}
-		for (int i = 0; i < curr_bd->alt_regs_size; i++) {
-			if (!strcasecmp(reg.c_str(), curr_bd->alt_regs[i].name) ||
-			    !strcasecmp(reg.c_str(), curr_bd->alt_regs[i].name + curr_bd->prefix)) {
-				return curr_bd->alt_regs[i].reg;
+		for (const auto &curr : curr_bd->alt_regs) {
+			if (!strcasecmp(reg.c_str(), curr.name) ||
+			    !strcasecmp(reg.c_str(), curr.name + curr_bd->prefix)) {
+				return curr.reg;
 			}
 		}
 	}
-	return strtoull(reg.c_str(), NULL, 0);
+	return strtoull(reg.c_str(), nullptr, 0);
 }
 
 static const char *reg_name(const struct board_list *curr_bd, unsigned long long reg)
 {
 	if (curr_bd) {
-		for (int i = 0; i < curr_bd->regs_size; i++) {
-			if (reg == curr_bd->regs[i].reg)
-				return curr_bd->regs[i].name;
+		for (const auto &curr : curr_bd->regs) {
+			if (reg == curr.reg)
+				return curr.name;
 		}
-		for (int i = 0; i < curr_bd->alt_regs_size; i++) {
-			if (reg == curr_bd->regs[i].reg)
-				return curr_bd->regs[i].name;
+		for (const auto &curr : curr_bd->regs) {
+			if (reg == curr.reg)
+				return curr.name;
 		}
 	}
-	return NULL;
+	return nullptr;
 }
 
 static const char *binary(unsigned long long val)
@@ -369,18 +302,18 @@ static int doioctl(int fd, unsigned long int request, void *parm, const char *na
 
 static int parse_subopt(char **subs, const char * const *subopts, char **value)
 {
-	int opt = getsubopt(subs, (char * const *)subopts, value);
+	int opt = v4l_getsubopt(subs, const_cast<char * const *>(subopts), value);
 
 	if (opt == -1) {
 		fprintf(stderr, "Invalid suboptions specified\n");
 		usage();
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
-	if (*value == NULL) {
+	if (*value == nullptr) {
 		fprintf(stderr, "No value given to suboption <%s>\n",
 				subopts[opt]);
 		usage();
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	return opt;
 }
@@ -399,7 +332,7 @@ int main(int argc, char **argv)
 	struct v4l2_dbg_register set_reg;
 	struct v4l2_dbg_register get_reg;
 	struct v4l2_dbg_chip_info chip_info;
-	const struct board_list *curr_bd = NULL;
+	const struct board_list *curr_bd = nullptr;
 	char short_options[26 * 2 * 3 + 1];
 	int idx = 0;
 	std::string reg_min_arg, reg_max_arg;
@@ -416,7 +349,7 @@ int main(int argc, char **argv)
 
 	if (argc == 1) {
 		usage();
-		return 0;
+		std::exit(EXIT_SUCCESS);
 	}
 	for (i = 0; long_options[i].name; i++) {
 		if (!isalpha(long_options[i].val))
@@ -429,7 +362,7 @@ int main(int argc, char **argv)
 			short_options[idx++] = ':';
 		}
 	}
-	while (1) {
+	while (true) {
 		int option_index = 0;
 
 		short_options[idx] = 0;
@@ -438,11 +371,23 @@ int main(int argc, char **argv)
 		if (ch == -1)
 			break;
 
-		options[(int)ch] = 1;
+		options[ch] = 1;
+		if (!option_index) {
+			for (i = 0; long_options[i].val; i++) {
+				if (long_options[i].val == ch) {
+					option_index = i;
+					break;
+				}
+			}
+		}
+		if (long_options[option_index].has_arg == optional_argument &&
+		    !optarg && argv[optind] && argv[optind][0] != '-')
+			optarg = argv[optind++];
+
 		switch (ch) {
 		case OptHelp:
 			usage();
-			return 0;
+			std::exit(EXIT_SUCCESS);
 
 		case OptSetDevice:
 			device = optarg;
@@ -457,12 +402,12 @@ int main(int argc, char **argv)
 		case OptChip:
 			if (!memcmp(optarg, "subdev", 6) && isdigit(optarg[6])) {
 				match.type = V4L2_CHIP_MATCH_SUBDEV;
-				match.addr = strtoul(optarg + 6, NULL, 0);
+				match.addr = strtoul(optarg + 6, nullptr, 0);
 				break;
 			}
 			if (!memcmp(optarg, "bridge", 6)) {
 				match.type = V4L2_CHIP_MATCH_BRIDGE;
-				match.addr = strtoul(optarg + 6, NULL, 0);
+				match.addr = strtoul(optarg + 6, nullptr, 0);
 				break;
 			}
 			match.type = V4L2_CHIP_MATCH_BRIDGE;
@@ -474,23 +419,23 @@ int main(int argc, char **argv)
 			break;
 
 		case OptGetRegister:
-			get_regs.push_back(optarg);
+			get_regs.emplace_back(optarg);
 			break;
 
 		case OptSetStride:
-			forcedstride = strtoull(optarg, 0L, 0);
+			forcedstride = strtoull(optarg, nullptr, 0);
 			break;
 
 		case OptListRegisters:
 			subs = optarg;
-			if (subs == NULL)
+			if (subs == nullptr)
 				break;
 
 			while (*subs != '\0') {
-				static const char * const subopts[] = {
+				static constexpr const char *subopts[] = {
 					"min",
 					"max",
-					NULL
+					nullptr
 				};
 
 				switch (parse_subopt(&subs, subopts, &value)) {
@@ -526,7 +471,7 @@ int main(int argc, char **argv)
 	if ((fd = open(device, O_RDWR)) < 0) {
 		fprintf(stderr, "Failed to open %s: %s\n", device,
 			strerror(errno));
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 
 	doioctl(fd, VIDIOC_QUERYCAP, &vcap, "VIDIOC_QUERYCAP");
@@ -535,16 +480,8 @@ int main(int argc, char **argv)
 	/* Information Opts */
 
 	if (options[OptGetDriverInfo]) {
-		printf("Driver info:\n");
-		printf("\tDriver name   : %s\n", vcap.driver);
-		printf("\tCard type     : %s\n", vcap.card);
-		printf("\tBus info      : %s\n", vcap.bus_info);
-		printf("\tDriver version: %d.%d.%d\n",
-				vcap.version >> 16,
-				(vcap.version >> 8) & 0xff,
-				vcap.version & 0xff);
-		printf("\tCapabilities  : 0x%08X\n", vcap.capabilities);
-		printf("%s", cap2s(vcap.capabilities).c_str());
+		printf("Driver Info:\n");
+		v4l2_info_capability(vcap);
 	}
 
 	chip_info.name[0] = '\0';
@@ -557,9 +494,9 @@ int main(int argc, char **argv)
 		if (!strncasecmp(match.name, "ac97", 4)) {
 			curr_bd = &boards[AC97_BOARD];
 		} else {
-			for (int board = ARRAY_SIZE(boards) - 1; board >= 0; board--) {
-				if (!strcasecmp(chip_info.name, boards[board].name)) {
-					curr_bd = &boards[board];
+			for (size_t board = boards.size(); board; board--) {
+				if (!strcasecmp(chip_info.name, boards[board - 1].name)) {
+					curr_bd = &boards[board - 1];
 					break;
 				}
 			}
@@ -570,8 +507,10 @@ int main(int argc, char **argv)
 
 	if (options[OptSetRegister]) {
 		set_reg.match = match;
-		if (optind >= argc)
+		if (optind >= argc) {
 			usage();
+			exit(1);
+		}
 		set_reg.reg = parse_reg(curr_bd, reg_set_arg);
 		while (optind < argc) {
 			unsigned size = 0;
@@ -580,7 +519,7 @@ int main(int argc, char **argv)
 				    "VIDIOC_DBG_G_REGISTER") >= 0)
 				size = set_reg.size;
 
-			set_reg.val = strtoull(argv[optind++], NULL, 0);
+			set_reg.val = strtoull(argv[optind++], nullptr, 0);
 			if (doioctl(fd, VIDIOC_DBG_S_REGISTER, &set_reg,
 						"VIDIOC_DBG_S_REGISTER") >= 0) {
 				const char *name = reg_name(curr_bd, set_reg.reg);
@@ -623,9 +562,8 @@ int main(int argc, char **argv)
 		get_reg.match = match;
 		printf("ioctl: VIDIOC_DBG_G_REGISTER\n");
 
-		for (std::vector<std::string>::iterator iter = get_regs.begin();
-				iter != get_regs.end(); ++iter) {
-			get_reg.reg = parse_reg(curr_bd, *iter);
+		for (const auto &reg : get_regs) {
+			get_reg.reg = parse_reg(curr_bd, reg);
 			if (ioctl(fd, VIDIOC_DBG_G_REGISTER, &get_reg) < 0)
 				fprintf(stderr, "ioctl: VIDIOC_DBG_G_REGISTER "
 						"failed for 0x%llx\n", get_reg.reg);
@@ -669,9 +607,9 @@ int main(int argc, char **argv)
 			else
 				reg_max = parse_reg(curr_bd, reg_max_arg);
 
-			for (int i = 0; i < curr_bd->regs_size; i++) {
-				if (reg_min_arg.empty() || ((curr_bd->regs[i].reg >= reg_min) && curr_bd->regs[i].reg <= reg_max)) {
-					get_reg.reg = curr_bd->regs[i].reg;
+			for (const auto &curr : curr_bd->regs) {
+				if (reg_min_arg.empty() || ((curr.reg >= reg_min) && curr.reg <= reg_max)) {
+					get_reg.reg = curr.reg;
 
 					if (ioctl(fd, VIDIOC_DBG_G_REGISTER, &get_reg) < 0)
 						fprintf(stderr, "ioctl: VIDIOC_DBG_G_REGISTER "
@@ -705,7 +643,7 @@ int main(int argc, char **argv)
 			goto list_done;
 		}
 
-		p = strchr(chip_info.name, ' ');
+		p = std::strchr(chip_info.name, ' ');
 		if (p)
 			*p = '\0';
 		name = chip_info.name;
@@ -747,7 +685,7 @@ list_done:
 		static char buf[40960];
 		int len = -1;
 
-		if (doioctl(fd, VIDIOC_LOG_STATUS, NULL, "VIDIOC_LOG_STATUS") == 0) {
+		if (doioctl(fd, VIDIOC_LOG_STATUS, nullptr, "VIDIOC_LOG_STATUS") == 0) {
 			printf("\nStatus Log:\n\n");
 #ifdef HAVE_KLOGCTL
 			len = klogctl(3, buf, sizeof(buf) - 1);
@@ -775,18 +713,18 @@ list_done:
 	}
 
 	if (options[OptListSymbols]) {
-		if (curr_bd == NULL) {
+		if (curr_bd == nullptr) {
 			printf("No symbols found for driver %s\n", vcap.driver);
 		}
 		else {
 			printf("Symbols for driver %s:\n", curr_bd->name);
-			for (int i = 0; i < curr_bd->regs_size; i++)
-				printf("0x%08x: %s\n", curr_bd->regs[i].reg, curr_bd->regs[i].name);
-			for (int i = 0; i < curr_bd->alt_regs_size; i++)
-				printf("0x%08x: %s\n", curr_bd->alt_regs[i].reg, curr_bd->alt_regs[i].name);
+			for (const auto &curr : curr_bd->regs)
+				printf("0x%08x: %s\n", curr.reg, curr.name);
+			for (const auto &curr : curr_bd->alt_regs)
+				printf("0x%08x: %s\n", curr.reg, curr.name);
 		}
 	}
 
 	close(fd);
-	exit(0);
+	std::exit(EXIT_SUCCESS);
 }
