@@ -86,6 +86,8 @@ constexpr val_def which_val_def[] = {
 	{ V4L2_CTRL_WHICH_CUR_VAL,	"V4L2_CTRL_WHICH_CUR_VAL" },
 	{ V4L2_CTRL_WHICH_DEF_VAL,	"V4L2_CTRL_WHICH_DEF_VAL" },
 	{ V4L2_CTRL_WHICH_REQUEST_VAL,	"V4L2_CTRL_WHICH_REQUEST_VAL" },
+	{ V4L2_CTRL_WHICH_MIN_VAL,	"V4L2_CTRL_WHICH_MIN_VAL" },
+	{ V4L2_CTRL_WHICH_MAX_VAL,	"V4L2_CTRL_WHICH_MAX_VAL" },
 	{ -1, "" }
 };
 
